/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package Simulation;

import Simulation.SimulationCreature.StakeholderType;
import SoarBridge.SoarBridge;
import SoarBridge.SoarBridgeException;
import SoarBridge.SoarCommand;
import SoarBridge.SoarCommandEat;
import SoarBridge.SoarCommandGet;
import SoarBridge.SoarCommandMove;
import SoarBridge.SoarCommandDeliver;
import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;
import org.apache.log4j.Logger;
import ws3dproxy.CommandExecException;
import ws3dproxy.CommandUtility;
import ws3dproxy.WS3DProxy;
import ws3dproxy.model.Creature;
import ws3dproxy.model.Thing;
import ws3dproxy.model.World;

/**
 *
 * @author Danilo
 */
public class SimulationTask
{
    Logger logger = Logger.getLogger(SimulationTask.class.getName());
    WS3DProxy proxy = null;
    //Workspace workspace = null;
    SoarBridge soarBridge = null;
    Creature c = null;
    World w = null;

    public SimulationTask()
    {
        proxy = new WS3DProxy();
        //proxy.connect();
    }

    /**
     * Initialize the Simulation robot and SOAR debugger
     * @param rulesPath The path in file system where the SOAR rules are located
     * @throws SoarBridgeException
     */
    public void initializeCreatureAndSOAR(String rulesPath, Boolean runDebugger, String soarDebuggerPath, Integer soarDebuggerPort) throws SoarBridgeException, CommandExecException
    {
        //robot's motor system is started and it is prepared to start moving
        //as soon as a wheel velocity command is sent.
        c = proxy.createCreature(100,100,0);
        w = proxy.getWorld();
        c.start();
        w.grow(1);

        // SOAR Enviroment
        soarBridge = new SoarBridge("agentSimulationSOAR", new File(rulesPath), runDebugger,soarDebuggerPath, soarDebuggerPort);
    }

    /**
     * Initialize the Simulation environment
     */
    public void initializeEnviroment(Boolean prepareEnviromentAndStartGame) throws CommandExecException
    {
        if (prepareEnviromentAndStartGame)
        {
            // Create Simulation Enviroment - Bricks
            CommandUtility.sendNewBrick(4,747.0,2.0,800.0,567.0);
            CommandUtility.sendNewBrick(4,50.0,-4.0,747.0,47.0);
            CommandUtility.sendNewBrick(4,49.0,562.0,796.0,599.0);
            CommandUtility.sendNewBrick(4,-2.0,6.0,50.0,599.0);            
        }
    }

    /**
     * Start Simulation Method
     * @throws SoarBridgeException
     * @throws NullPointerException
     */
    public void runSimulation() throws SoarBridgeException, NullPointerException, IllegalAccessException, CommandExecException
    {
        if (soarBridge != null)
        {
            c.updateState();
            //Status currentStatus = c.getState();
            List<Thing> v = c.getThingsInVision();
            //logger.info("Objetos no Ambiente: "+v.size());
            //System.out.println("Objetos no Ambiente: "+v.size());
            for (Thing t : v) {
                logger.info(t.toString());
            }
            

            if (c != null)
            {
                // Prepare Creature To Simulation
                prepareAndSetupCreatureToSimulation(c, v);

                // Run simulation
                soarBridge.runSimulation();

                // Process Responde Commands
                processResponseCommands();
            }
            else
            {
                throw new NullPointerException("There are no creatures in simulation");
            }
        }
        else
        {
            throw new NullPointerException("soarBrige is null. Please, invoke initializeEnviroment");
        }
    }

    private void prepareAndSetupCreatureToSimulation(Creature creature, List<Thing> things) throws SoarBridgeException, IllegalAccessException
    {
        if (creature != null)
        {
            // Create Visual Sensor
            VisualSensor visualSensor = new VisualSensor();
            visualSensor.setSensorReadings(things);

            // Create Fuel Sensor
            FuelSensor fuelSensor = new FuelSensor();
            FuelSensorReadings fuelSensorReadings = new FuelSensorReadings();
            fuelSensorReadings.setCurrentFuel(creature.getFuel());
            fuelSensor.setSensorReadings(fuelSensorReadings);

            // Put things in SOAR output interface to start simulation
            SimulationRobot simulationCreature = new SimulationRobot(creature);
            simulationCreature.addSensorsAvailable(visualSensor);
            simulationCreature.addSensorsAvailable(fuelSensor);
            
            // @author: ra036647 - make vector available and filled
            simulationCreature.leafletList = new Vector(creature.getLeaflets());
            
            // Setup StackHolder for run simulation
            soarBridge.setupStackHolder(StakeholderType.CREATURE,simulationCreature);
        }
        else
        {
            throw new IllegalArgumentException("Arguments ares null");
        }
    }

    /***
     * Stop Simulation
     */
    public void stopSimulation() throws CommandExecException
    {
        c.stop();
    }

    private void processResponseCommands() throws SoarBridgeException, CommandExecException
    {
        // get simulation results
        ArrayList<SoarCommand> commandList = soarBridge.getReceivedCommands();

        if (commandList != null)
        {
            for (SoarCommand command:commandList)
            {
                switch (command.getCommandType())
                {
                    case MOVE:
                        processMoveCommand((SoarCommandMove)command.getCommandArgument());
                    break;

                    case GET:
                        processGetCommand((SoarCommandGet)command.getCommandArgument());
                    break;

                    case EAT:
                        processEatCommand((SoarCommandEat)command.getCommandArgument());
                    break;

                    case DELIVER: // @author: ra036647 - deliver command
                        processDeliverCommand((SoarCommandDeliver)command.getCommandArgument());
                    break;

                    default:
                        // Do nothing
                    break;
                }
            }
        }
    }

    /**
     * Send Move Command to World Server
     * @param soarCommandMove Soar Move Command Structure
     */
    private void processMoveCommand(SoarCommandMove soarCommandMove) throws CommandExecException
    {
        if (soarCommandMove != null)
        {
            if (soarCommandMove.getX() != null && soarCommandMove.getY() != null)
            {
                CommandUtility.sendGoTo("0", soarCommandMove.getRightVelocity(), soarCommandMove.getLeftVelocity(), soarCommandMove.getX(), soarCommandMove.getY());
            }
            else
            {
                CommandUtility.sendSetTurn("0",soarCommandMove.getLinearVelocity(),soarCommandMove.getRightVelocity(),soarCommandMove.getLeftVelocity());
            }
        }
        else
        {
            throw new NullPointerException("soarCommand is null");
        }
    }

    /**
     * Send Get Command to World Server
     * @param soarCommandGet Soar Get Command Structure
     */
    private void processGetCommand(SoarCommandGet soarCommandGet) throws CommandExecException
    {
        if (soarCommandGet != null)
        {
            c.putInSack(soarCommandGet.getThingName());
        }
        else
        {
            throw new NullPointerException("soarCommand is null");
        }
    }

     /**
     * Send Eat Command to World Server
     * @param soarCommandEat Soar Eat Command Structure
     */
    private void processEatCommand(SoarCommandEat soarCommandEat) throws CommandExecException
    {
        if (soarCommandEat != null)
        {
            c.eatIt(soarCommandEat.getThingName());
        }
        else
        {
            throw new NullPointerException("soarCommand is null");
        }
    }
    
    public static boolean Delivered = false;
    
    // @author: ra036647 - deliver command
    private void processDeliverCommand(SoarCommandDeliver soarCommandDeliver) throws CommandExecException
    {
        if (soarCommandDeliver != null)
        {
            if (soarCommandDeliver.getTotalPayment() != null)
            {
                SimulationTask.Delivered = true;
                System.out.println("Simulation ended with payment of " + soarCommandDeliver.getTotalPayment().toString());
            }
        }
        else
        {
            throw new NullPointerException("soarCommand is null");
        }
    }
}
